#include <v8.h>
#include <node.h>
#include <PDL.h>


using namespace v8;
using namespace node;

namespace node_pdl {

//  static Persistent<Object> functionMap;
/*
  static Handle<Value> parseError(PDL_Err err) {
    String message;
    switch (err) {
      case PDL_NOERROR: return Undefined();
      case PDL_EMEMORY: message = String::New("PDL_EMEMORY"); break;
      case PDL_ECONNECTION: message = String::New("PDL_ECONNECTION"); break;
      case PDL_INVALIDINPUT: message = String::New("PDL_EMEMORY"); break;
      case PDL_EOTHER: message = String::New("PDL_EOTHER"); break;
      case PDL_UNINIT: message = String::New("PDL_UNINIT"); break;
      case PDL_NOTALLOWED: message = String::New("PDL_NOTALLOWED"); break;
      case PDL_LICENSEFAILURE: message = String::New("PDL_LICENSEFAILURE"); break;
      case PDL_STRINGTOOSMALL: message = String::New("PDL_STRINGTOOSMALL"); break;
      case PDL_SYSTEMERROR_FILE: message = String::New("PDL_SYSTEMERROR_FILE"); break;
      case PDL_SYSTEMERROR_NET: message = String::New("PDL_SYSTEMERROR_NET"); break;
      case PDL_APPINFO: message = String::New("PDL_APPINFO"); break;
    }
    Exception::Error err = Exception::Error(message);
    err.Set(String::New("PDL_CODE", Number::New(err)));
    return ThrowException(err);
  }
*/

  static Handle<Value> ThrowPDLError(PDL_Err err, const char* name) {
    return ThrowException(Exception::Error(String::Concat(
      String::Concat(
        String::New(name),
        String::New(": ")
      ),
      String::New(PDL_GetError())
    )));
  }

  static Handle<Value> Init(const Arguments& args) {
    HandleScope scope;
    int flags = (args[0]->Int32Value());
    PDL_Err err = PDL_Init(flags);
    if (err != PDL_NOERROR) return ThrowPDLError(err, __func__);
    return Undefined();
  }

  static Handle<Value> Quit(const Arguments& args) {
    HandleScope scope;
    PDL_Quit();
    return Undefined();
  }

  static Handle<Value> GetHardwareID(const Arguments& args) {
    HandleScope scope;
    return Number::New(PDL_GetHardwareID());
  }

  static Handle<Value> GetUniqueID(const Arguments& args) {
    HandleScope scope;
    char buffer[64];
    PDL_Err err = PDL_GetUniqueID(buffer, 64);
    if (err != PDL_NOERROR) return ThrowPDLError(err, __func__);
    return String::New((char*) buffer);
  }

  static Handle<Value> SetFirewallPortStatus(const Arguments& args) {
    HandleScope scope;
    int port = args[0]->Int32Value();
    bool open = args[1]->BooleanValue();
    PDL_Err err = PDL_SetFirewallPortStatus(port, (PDL_bool)open);
    if (err != PDL_NOERROR) return ThrowPDLError(err, __func__);
    return Undefined();
  }

  static Handle<Value> SetKeyboardState(const Arguments& args) {
    HandleScope scope;
    bool visible = args[0]->BooleanValue();
    PDL_Err err = PDL_SetKeyboardState((PDL_bool)visible);
    if (err != PDL_NOERROR) return ThrowPDLError(err, __func__);
    return Undefined();
  }
/*
  static Handle<Value> JSRegistrationComplete(const Arguments& args) {
    HandleScope scope;
    PDL_Err err = PDL_JSRegistrationComplete();
    if (err != PDL_NOERROR) {
      return ThrowException(Exception::Error(String::Concat(
        String::New("JSRegistrationComplete failed!: "),
        String::New("PDL ERROR") // TODO: give better error reporting
      )));
    }
    return Undefined();
  }

  static Handle<Value> CallJS(const Arguments& args) {
    HandleScope scope;

    String::Utf8Value name(args[0]);
    String::Utf8Value json(args[1]);

    PDL_CallJS(*name, NULL, 0);
    return Undefined();
    
  }

  PDL_bool JSRouter(PDL_JSParameters *params) {
    printf("ME\n");
    // TODO: get real name, assume "foo" for now
    const char* name = "foo";
    
    const int numParams = PDL_GetNumJSParams(params);
    
    Handle<Value> argv[numParams];
    for (int i = 0; i < numParams; i++) {
      argv[i] = String::New(PDL_GetJSParamString(params, i));
    }

    Handle<Function>::Cast(functionMap->Get(String::New(name)))->Call(functionMap, numParams, argv);
    return PDL_TRUE;
  }

  static Handle<Value> RegisterJSHandler(const Arguments& args) {
    HandleScope scope;

    String::Utf8Value name(args[0]);
    functionMap->Set(args[0], args[1]);
    PDL_RegisterJSHandler(*name, JSRouter);
    // TODO: handle error

    return Undefined();
  }
*/
 
}


extern "C" void
init(Handle<Object> target)
{
  HandleScope scope;
  
//  node_pdl::functionMap = Persistent<Object>::New(Object::New());

  // Hardware Types
  target->Set(String::New("HARDWARE_UNKNOWN"), Number::New(HARDWARE_UNKNOWN));
  target->Set(String::New("HARDWARE_PRE"), Number::New(HARDWARE_PRE));
  target->Set(String::New("HARDWARE_PRE_PLUS"), Number::New(HARDWARE_PRE_PLUS));
  target->Set(String::New("HARDWARE_PIXI"), Number::New(HARDWARE_PIXI));
  target->Set(String::New("HARDWARE_VEER"), Number::New(HARDWARE_VEER));
  target->Set(String::New("HARDWARE_PRE_2"), Number::New(HARDWARE_PRE_2));
  target->Set(String::New("HARDWARE_PRE_3"), Number::New(HARDWARE_PRE_3));
  target->Set(String::New("HARDWARE_TOUCHPAD"), Number::New(HARDWARE_TOUCHPAD));

  // Custom User Events
  target->Set(String::New("GPS_UPDATE"), Number::New(PDL_GPS_UPDATE));
  target->Set(String::New("GPS_FAILURE"), Number::New(PDL_GPS_FAILURE));
  target->Set(String::New("COMPASS"), Number::New(PDL_COMPASS));

  // Bound functions
  NODE_SET_METHOD(target, "init", node_pdl::Init);
  NODE_SET_METHOD(target, "quit", node_pdl::Quit);
  NODE_SET_METHOD(target, "getHardwareID", node_pdl::GetHardwareID);
  NODE_SET_METHOD(target, "getUniqueID", node_pdl::GetUniqueID);
  NODE_SET_METHOD(target, "setFirewallPortStatus", node_pdl::SetFirewallPortStatus);
  NODE_SET_METHOD(target, "setKeyboardState", node_pdl::SetKeyboardState);
//  NODE_SET_METHOD(target, "jsRegistrationComplete", node_pdl::JSRegistrationComplete);
//  NODE_SET_METHOD(target, "callJS", node_pdl::CallJS);
//  NODE_SET_METHOD(target, "registerJSHandler", node_pdl::RegisterJSHandler);
  
}




